#
# gSmile - emoticon manager
# 
# Copyright (C) 2005, Federico Pelloni <federico.pelloni@gmail.com>
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#


from xml.dom import minidom, Node
from xml.parsers.expat import ExpatError
from gtk import FALSE, TRUE, RESPONSE_YES
from time import time
from os.path import normpath, isdir
from os import mkdir
from gsmile.shared import gglobals, quit_program
from folder import Folder
from emoticon import Emoticon
from gsmile.gui import dialog


class Database:
	
	def __init__(self, database):
		"""Init the database class"""		

		self.database = database
		self.ok		= 1	
		self.dbfile	= normpath(database + "/database.xml")
		self.imgdir	= normpath(database + "/emoticons/")
		
		gglobals.db = self
		
		
		
	def start(self):
		"""Start the database class, parsing the database and loading GUI and database modules"""
			
		if not isdir(self.database):
		
			try:
				mkdir(self.database)

			except:
				print _("ERROR: Unable to create the database directory")
				dialog.error(_("Unable to create the database directory"),
						_("gSmile wasn't able to create the database directory, exiting"))
				quit_program(1, True)
				self.ok = 0
				return
		
		try:
			self.db = minidom.parse(self.dbfile)
			
		except IOError:
			print _("WARNING: Database not found.")
			r = dialog.question(_("Database not found!"), 
										 _("Should gSmile build it?"))

			if r == RESPONSE_YES:
				print _("Going to build it...")
				self.build ()
			else:
				print _("No database: exiting...")
				dialog.error(_("No database, unable to run"),
									 _("You chose not to build the database, gSmile can't work without it, so it will exit.") )
				quit_program(1, True)
				self.ok = 0
				return
			
		except ExpatError:
			print _("ERROR: Database corrupted!\n" \
					"Should gSmile rebuild it?" )
			r = dialog.question(_("Database corrupted!"), 
										 _("The database looks corrupted, should gSmile rebuild it?\n(The file will be overwritten)"))

			if r == RESPONSE_YES:
				print _("Going to rebuild it...")
				self.build ()
			else:
				print _("Database corrupted: exiting...")
				dialog.error(_("Database corrupted, unable to run"),
									 _("Your database looks wrong, and you chose not to rebuild it: gSmile can't work without a correct database, so it will exit.") )
				quit_program(1, True)
				self.ok = 0
				return

		if not isdir(self.imgdir):
		
			try:
				mkdir(self.imgdir)

			except:
				print _("ERROR: Unable to create emoticons directory")
				dialog.error(_("Unable to create emoticons directory"),
						_("gSmile wasn't able to create the emoticons directory, exiting"))
				quit_program(1, True)
				self.ok = 0
				return

		self.db.firstChild.normalize()

		self.load()
		
			



	def build(self):
		"""Build a new empty database"""
		
		xmldb	= minidom.getDOMImplementation ()
		
		newdoc	= xmldb.createDocument (None, "emoticondb", None)
		
		try:
			file = open (self.dbfile, 'w')
			newdoc.writexml(file)
			file.close ()
			newdoc.unlink()
		except:
			newdoc.unlink()
			print _("ERROR: Unable to write database file. " \
					"Maybe wrong permissions for the current user")
			quit_program (1)
			
		self.start()
			
			
	
	def load(self):
		"""Start loading folders and emoticons from database"""
		
		root = self.db.firstChild
		
		self.emoticons = {}
		self.folders = {}
#		self._top10s = {
#			"most used": {
#				"min":	0,
#				"max":	0,
#				"items": []
#			},
#			"less used": {
#				"min":	0,
#				"max":	0,
#				"items": []
#			},
#			"most recently used": {
#				"min":	0,
#				"max":	0,
#				"items": []
#			},
#			"most recently added": {
#				"min":	0,
#				"max":	0,
#				"items": []
#			},
#		}
		
		self._last_emoticon_id = 0
		self._last_folder_id = 0
		
		self.folders[0] = Folder(root, None, 0)
		self.folders[0].set_iter(None)

		for i in root.childNodes:
		
			if i.nodeName == 'folder':
			
				self.parse_folder(i, self.folders[0])
		
		
		
	def parse_folder(self, f, parent):
		"""Parse a folder from an XML node of the database"""
	
		self._last_folder_id += 1
		
		folderid = self._last_folder_id
		
		self.folders[folderid] = Folder(f, parent, folderid)
		self.folders[folderid].create_iter()
		
		for i in f.childNodes:
		
			if i.nodeName == 'folder':
			
				self.parse_folder(i, self.folders[folderid])
				
			elif i.nodeName == 'emoticon':
			
				self.parse_emoticon(i, folderid)
				
		return
				
				
	def parse_emoticon(self, e, fid):
		"""Parse an emoticon from the database"""
	
		self._last_emoticon_id += 1
		
		ei = Emoticon(e, fid, self._last_emoticon_id)
		self.emoticons[self._last_emoticon_id] = ei
												 	
		return
		
				
				
	def show_folder(self, folder):
		"""Callback for showing a folder when selected"""
		
		if folder > 0:
		
			self.folders[folder].show()
				
	
	
	def new_folder(self, parent_id):
		"""Create a new folder"""
	
		created = str(int(time()))
		
		parent_node = self.folders[parent_id]
		
		folder_node = self.db.createElement('folder')
		parent_node.ref.appendChild(folder_node)
		
		fn_name = self.db.createElement('name')
		fn_name.appendChild(self.db.createTextNode("New folder"))
		folder_node.appendChild(fn_name)
		
		fn_created = self.db.createElement('created')
		fn_created.appendChild(self.db.createTextNode(created))
		folder_node.appendChild(fn_created)
		
		self.parse_folder(folder_node, parent_node)
		
		return self.folders[self._last_folder_id]
	
	
	
	def update_folder(self, folder_id, name):
		"""Callback for updating the name of a folder"""
	
		folder = self.folders[folder_id]
		
		folder.set('name', name, 1)
		
		return
	
	
	
	def delete_folder(self, folder):
		"""Callback to delete a folder"""
	
		self.folders[folder].remove()
	
	
	
	def move_folder_up(self, folder_id):
		"""Move a folder up a position"""
		
		self.folders[folder_id].move_up()
		
		return
		
		
	
	def move_folder_down(self, folder_id):
		"""Move a folder down a position"""
		
		self.folders[folder_id].move_down()
		
		return
		
		
	
	def new_emoticon(self, folder_id, infos):
		"""Callback: new emoticon"""
	
		inserted = str(int(time()))
		last_used = '0'			
		
		folder_node = self.folders[folder_id].ref
		
		em_node = self.db.createElement('emoticon')
		folder_node.appendChild(em_node)
		
		en_name = self.db.createElement('name')
		en_name.appendChild(self.db.createTextNode(infos['name']))
		em_node.appendChild(en_name)
		
		en_descr = self.db.createElement('description')
		en_descr.appendChild(self.db.createTextNode(infos['descr']))
		em_node.appendChild(en_descr)
		
		en_url = self.db.createElement('url')
		en_url.appendChild(self.db.createTextNode(infos['url']))
		em_node.appendChild(en_url)
		
		en_file = self.db.createElement('file')
		en_file.appendChild(self.db.createTextNode('%s.png' % inserted))
		em_node.appendChild(en_file)
			
		en_inserted = self.db.createElement('inserted')
		en_inserted.appendChild(self.db.createTextNode(inserted))
		em_node.appendChild(en_inserted)
		
		en_used = self.db.createElement("used")
		en_used.appendChild(self.db.createTextNode("0"))
		em_node.appendChild(en_used)
		
		en_lu = self.db.createElement('last_used')
		en_lu.appendChild(self.db.createTextNode(last_used))
		em_node.appendChild(en_lu)
		
		infos['pixbuf'].save('%s/%s.png' % (self.imgdir, inserted), 'png')
				
		self.parse_emoticon(em_node, folder_id)

		self.show_folder(folder_id)
											
		return
	

#
#	NOTE: this function is disabled until I find out a quick way to build Top10s
#		  and I work on it
#
#	def do_top10(self, mode):
#	
#		if mode == None:
#			# No top10 selected
#			return
#			
#		list = {}
#		
#		if mode == "most used":
#					
#			for e in self.emoticons.itervalues():
#			
#				used = int(e.get("used"))
#		
#				if used > 0:
#				
#					if len(list) < 10:
#					
#						list[e] = used
#						
#					else:
#					
#						min = min(list.values)
#					
#						if min < used:
#						
#							list[e] = used
#
#							list.remove(min)
#			
#			for e in list.keys():
#			
#				e.show()
#		
#		
#		if mode == "most recently used":
#					
#			for e in self.emoticons.itervalues():
#			
#				last_used = int(e.get("last_used"))
#		
#				if last_used > 0:
#				
#					if len(list) < 10:
#					
#						list[e] = last_used
#						
#					else:
#					
#						min = min(list.values)
#					
#						if min < last_used:
#						
#							list[e] = last_used
#						
#							list.remove(min)
#			
#			for e in list.keys():
#			
#				e.show()
#				
#				
#		if mode == "most recently added":
#					
#			for e in self.emoticons.itervalues():
#			
#				inserted = int(e.get("inserted"))
#		
#				if len(list) < 10:
#				
#					list[inserted] = e
#						
#				else:
#					
#					min = min(list.keys)
#
#					if min < inserted:
#						
#						list[inserted] = e
#						
#						try:
#							del list[min]
#						except:
#							pass
#			
#			for e in list.values():
#			
#				e.show()
#		
#	

		
	def close(self):
		"""Database dumping and closing operations"""
	
		ok = 0
	
		try:
			self.db.firstChild.normalize()
			f = open(self.dbfile, 'w')
			self.db.writexml(f)
			f.close()
			self.db.unlink()
			ok = 1
		except:
			print _("ERROR: Unable to write database file." \
					"Maybe wrong permissions for the current user")
			r = dialog.question(_("Unable to save the database, exit anyway?"),
						_("gSmile wasn't able to save successfully the database file, recent changes might be lost. Exit anyway?"))

		if r == RESPONSE_YES:
		
			ok = 1
			self.db.unlink()
			
		return ok
